#ifndef EVAL_HPP
#define EVAL_HPP

#include <cstdio>
#include <cstdlib>
#include <stdint.h>
#include <string>
#include <map>
#include <list>
#include <vector>
#include <set>
#include <iostream>
#include <sstream>
#include <algorithm>
#include "tree.hpp"
#include "util_tree.hpp"
#include "btree.hpp"
#include "util_btree.hpp"
using namespace std;

namespace aed {

  typedef int (*map_fun_t)(int);
  typedef int(*assoc_fun_t)(int, int);
  int square(int x) { return x*x; }
  int cube(int x) { return x*x*x; }
  int div2(int x) { return x/2; }
  int neg(int x) { return -x; }

  int suma(int a, int b) { return a+b; }
  int max(int a, int b) { return (a>b? a : b); }
  int min(int a, int b) { return (a<b? a : b); }

  // Simple hasher class for the examples
  class hasher_t  {
  private:
    // BJ hash (Bob Jenkins). From
    // http://www.chasanc.com/old/hashing_func.htm
    typedef uint32_t u4; /* unsigned 4-byte type */
    typedef uint8_t u1;   /* unsigned 1-byte type */
    u4 state;

    /* The mixing step */
#define mix(a,b,c)                              \
    {                                           \
      a=a-b;  a=a-c;  a=a^(c>>13);              \
      b=b-c;  b=b-a;  b=b^(a<<8);               \
      c=c-a;  c=c-b;  c=c^(b>>13);              \
      a=a-b;  a=a-c;  a=a^(c>>12);              \
      b=b-c;  b=b-a;  b=b^(a<<16);              \
      c=c-a;  c=c-b;  c=c^(b>>5);               \
      a=a-b;  a=a-c;  a=a^(c>>3);               \
      b=b-c;  b=b-a;  b=b^(a<<10);              \
      c=c-a;  c=c-b;  c=c^(b>>15);              \
    }

    /* The whole new hash function */
    u4 bjhash(register u1* k,u4 length, u4 initval)
    // register u1 *k;        /* the key */
    // u4           length;   /* the length of the key in bytes */
    // u4           initval;  /* the previous hash, or an arbitrary value */
    {
      register u4 a,b,c;  /* the internal state */
      u4          len;    /* how many key bytes still need mixing */

      /* Set up the internal state */
      len = length;
      a = b = 0x9e3779b9;  /* the golden ratio; an arbitrary value */
      c = initval;         /* variable initialization of internal state */

      /*---------------------------------------- handle most of the key */
      while (len >= 12)
        {
          a=a+(k[0]+((u4)k[1]<<8)+((u4)k[2]<<16) +((u4)k[3]<<24));
          b=b+(k[4]+((u4)k[5]<<8)+((u4)k[6]<<16) +((u4)k[7]<<24));
          c=c+(k[8]+((u4)k[9]<<8)+((u4)k[10]<<16)+((u4)k[11]<<24));
          mix(a,b,c);
          k = k+12; len = len-12;
        }

      /*------------------------------------- handle the last 11 bytes */
      c = c+length;
      switch(len)              /* all the case statements fall through */
        {
        case 11: c=c+((u4)k[10]<<24);
        case 10: c=c+((u4)k[9]<<16);
        case 9 : c=c+((u4)k[8]<<8);
          /* the first byte of c is reserved for the length */
        case 8 : b=b+((u4)k[7]<<24);
        case 7 : b=b+((u4)k[6]<<16);
        case 6 : b=b+((u4)k[5]<<8);
        case 5 : b=b+k[4];
        case 4 : a=a+((u4)k[3]<<24);
        case 3 : a=a+((u4)k[2]<<16);
        case 2 : a=a+((u4)k[1]<<8);
        case 1 : a=a+k[0];
          /* case 0: nothing left to add */
        }
      mix(a,b,c);
      /*-------------------------------------------- report the result */
      return c;
    }


  public:
    hasher_t() { reset(); }
    void reset() { state = 0L; }
    void hash(vector<int> w) { hash(&w[0],w.size()); }
    void hash(list<int> &L) {
      list<int>::iterator q = L.begin();
      while (q!=L.end()) hash(*q++);
    }
    void hash(vector< list<int> > &VL) {
      int N = VL.size();
      for (int j=0; j<N; j++) {
        list<int> &L = VL[j];
        hash(L.size());
        hash(L);
      }
    }

    void hash(int w) {
      state = bjhash((u1*)&w,sizeof(int),state);
    }

    void hash(int *w,int n) {
      for (int j=0; j<n; j++) hash(w[j]);
    }

    uint64_t val() {
      return state;
    }


  };

  // Simple random generator class for the examples
  class randomg_t  {
  private:
    hasher_t hasher;
    uint64_t state;
  public:
    randomg_t() { reset(0); }
    void reset(uint64_t seed=0) { hasher.reset(); state=seed; }
    uint64_t rand() { 
      hasher.hash(state++); 
      return hasher.val();
    }
  };

  
  //---:---<*>---:---<*>---:---<*>---:---<*>---:---<*>
  class Eval {

    //---:---<*>---:---<*>---:---<*>---:---<*>---:---<*>
    void dump(list<int> &L,string s="") {
      cout << s; 
      list<int>::iterator q = L.begin();
      while (q!=L.end()) cout << *q++ << " ";
      cout << endl;
    }

    //---:---<*>---:---<*>---:---<*>---:---<*>---:---<*>
    void dump(vector<int> &V,string s="") {
      cout << s; 
      vector<int>::iterator q = V.begin();
      while (q!=V.end()) cout << *q++ << " ";
      cout << endl;
    }

    //---:---<*>---:---<*>---:---<*>---:---<*>---:---<*>
    void dump(vector< list<int> > &VL,string s="") {
      int N = VL.size();
      // cout << "vector length " << N << endl;
      for (int j=0; j<N; j++) {
        cout << s << "[" << j << "]: ";
        dump(VL[j]);
      }
    }

    //---:---<*>---:---<*>---:---<*>---:---<*>---:---<*>
    void dump(set<int> &S,string s="") {
      cout << s; 
      set<int>::iterator q = S.begin();
      while (q!=S.end()) cout << *q++ << " ";
      cout << endl;
    }

    //---:---<*>---:---<*>---:---<*>---:---<*>---:---<*>
    void dump(vector< set<int> > &VS,string s="") {
      int N = VS.size();
      for (int j=0; j<N; j++) {
        cout << s << "[" << j << "]: ";
        dump(VS[j]);
      }
    }

    //---:---<*>---:---<*>---:---<*>---:---<*>---:---<*>
    void dump(map<int,int> &M,string s="") {
      cout << s << endl; 
      map<int,int>::iterator itm = M.begin();
      while (itm != M.end()){
        cout << "M[" << itm->first <<"] = "<< itm->second << endl;
        itm++;
      }
      cout << endl;
    }

    void printmap(map<int, list<bool> >& M) {
      cout << "M = {";
      map<int, list<bool> >::iterator p = M.begin();
      while (p!=M.end()) {
        cout << p->first << "->[";
        list<bool> &L = p->second;
        list<bool>::iterator q = L.begin();
        while (q!=L.end()) {
          cout << (*q ? "true" : "false");
          q++;
          if (q!=L.end()) cout << ", ";
        }
        cout << "]";
        p++;
        if (p!=M.end()) cout << ", ";
      }
      cout << "}" << endl;
    }

    list< list<int> > colas(list<int> &L) {
      list< list<int> > R;
      if (L.empty()) return R;
      list<int>::iterator it = L.begin();
      while (it != L.end()) {
        R.push_back(list<int>(it, L.end()));
        ++ it;
      }

      return R;
    }

    // Retorna todos los elementos de la lista
    // e.g.: "1 2 3 4 3 2 1" -> (1 2 3 4 3 2 1)
    void s2l(string s, list<int> &L) {
      istringstream is(s);
      int n;
      while (is >> n)
        L.push_back(n);
    }

    void s2v(string s, vector<int> &V) {
      istringstream is(s);
      int n;
      while (is >> n) V.push_back(n);
    }

    void prl(list<int> &L) {
      if (L.empty()) return;
      list<int>::iterator il = L.begin();
      for (; &*il != &*L.rbegin(); il ++)
        cout << *il << " ";
      cout << *il;
    }


    void s2m(string s, map<int, list<bool> > &M) {
      M.clear();
      istringstream is(s);

      int k, n, v;
      while (is >> k >> n) {
        list<bool> L;

        for (int i = 0 ; i < n ; i ++) {
          is >> v;
          L.push_back(v);
        }
        M[k] = L;
      }
    }

    void s2m(string s, map<int, list<int> > &M) {
      M.clear();
      istringstream is(s);
      int k, n, v;
      while (is >> k >> n) {
        list<int> L;
        for (int i = 0 ; i < n ; i ++) {
          is >> v;
          L.push_back(v);
        }
        M[k] = L;
      }
    }

    void s2m(string s, map<int, int> &M) {
      M.clear();
      istringstream is(s);
      int k, n, v;
      while (is >> k >> n) {
        M[k] = n;
      }
    }

    void s2s(string s, set<int> &S) {
      list<int> L;
      s2l(s,L);
      S.clear();
      S.insert(L.begin(),L.end());
    }

    // string -> vector de listas
    // (n1 L1 n2 L2 ...)
    // "3 1 2 3 4 5 4 3 2"  -> [(1 2 3) (5 4 3 2)]
    void s2vl(string s, vector< list<int> > &v) {
      istringstream is(s);
      int n1, n2;
      while (is >> n1) {
        v.push_back(list<int>());
        for (int i = 0 ; i < n1 ; i ++) {
          is >> n2;
          v.back().push_back(n2);
        }
      }
    }

    void s2vs(string s, vector< set<int> > &v) {
      istringstream is(s);
      int n1, n2;
      while (is >> n1) {
        v.push_back(set<int>());
        for (int i = 0 ; i < n1 ; i ++) {
          is >> n2;
          v.back().insert(n2);
        }
      }
    }

    void acum_hasta(list<int> &L, int n) {
      int acum = 0;
      list<int>::iterator it = L.begin();
      while (it != L.end()) {
        acum += *it;
        it = L.erase(it);
        if (acum >= n) {
          L.insert(it, acum);
          acum = 0;
        }
      }
      if (acum > 0) L.push_back(acum);
    }

    int readnum(string &s, unsigned int &n) {
      int k = 0;
      bool negativo = false;
      if (s[n] == '-') {
        negativo = true;
        n ++;
      }

      while (s[n] >= '0' && s[n] <= '9') {
        k *= 10;
        k += s[n] - '0';
        ++ n;
      }
      if (negativo) k *= -1;
      return k;
    }

    void lisp2tree(string s, tree<int> &T, unsigned int &i, 
                   tree<int>::iterator it) {
      while (i < s.size()) {
        while (s[i] == ' ') ++ i;
        
        if (s[i] == '(') {
          ++ i;
          while (s[i] == ' ') ++ i;
          it = T.insert(it, readnum(s, i));
          lisp2tree(s, T, i, it.lchild());
          ++ it;
        } else if (s[i] == ')') {
          ++ i;
          return;
        } else {
          it = T.insert(it, readnum(s, i));
          ++ it;
        }
      }
    }

    void preorder(tree<int> &T,tree<int>::iterator n,list<int> &L) {
      L.insert(L.end(),*n);
      tree<int>::iterator c = n.lchild();
      while (c!=T.end()) {
        preorder(T,c,L);
        c = c.right();
      }
    }

    void preorder(tree<int> &T,list<int> &L) {
      if (T.begin()==T.end()) return;
      preorder(T,T.begin(),L);
    }

    void postorder(tree<int> &T,tree<int>::iterator n,list<int> &L) {
      tree<int>::iterator c = n.lchild();
      while (c!=T.end()) {
        postorder(T,c,L);
        c = c.right();
      }
      L.insert(L.end(),*n);
    }

    void postorder(tree<int> &T,list<int> &L) {
      if (T.begin()==T.end()) return;
      postorder(T,T.begin(),L);
    }

    void lisp2btree(string s, btree<int> &T, unsigned int &i, 
                    btree<int>::iterator it) {
      while (i < s.size()) {
        while (s[i] == ' ') ++ i;

        if (s[i] == '(') {
          ++ i;
          it = T.insert(it, readnum(s, i));

          lisp2btree(s, T, i, it.left());
          lisp2btree(s, T, i, it.right());
        } else if (s[i] == ')' || s[i] == '.') {
          ++ i;
          return;
        } else {
          T.insert(it, readnum(s, i));
          return;
        }
      }
    }

    void lisp2btree(string s, btree<int> &T) {
      unsigned int i = 0;
      lisp2btree(s, T, i, T.begin());
    }

    void preorder(btree<int> &T,btree<int>::iterator n,list<int> &L) {
      if(n==T.end()) return;
      L.insert(L.end(),*n);
      preorder(T,n.left(),L);
      preorder(T,n.right(),L);
    }

    void preorder(btree<int> &T,list<int> &L) {
      if (T.begin()==T.end()) return;
      preorder(T,T.begin(),L);
    }

    void postorder(btree<int> &T,btree<int>::iterator n,list<int> &L) {
      if(n==T.end()) return;
      postorder(T,n.left(),L);
      postorder(T,n.right(),L);
      L.insert(L.end(),*n);
    }

    void postorder(btree<int> &T,list<int> &L) {
      if (T.begin()==T.end()) return;
      postorder(T,T.begin(),L);
    }

    void hash(hasher_t &hasher,tree<int> &T) {
      list<int> L;
      preorder(T,L);
      list<int>::iterator q=L.begin();
      while (q!=L.end()) hasher.hash(*q++);

      L.clear();
      postorder(T,L);
      q=L.begin();
      while (q!=L.end()) hasher.hash(*q++);
    }

    void hash(hasher_t &hasher,btree<int> &T) {
      list<int> L;
      preorder(T,L);
      list<int>::iterator q=L.begin();
      while (q!=L.end()) hasher.hash(*q++);

      L.clear();
      postorder(T,L);
      q=L.begin();
      while (q!=L.end()) hasher.hash(*q++);
    }

    void hash(hasher_t &hasher,set<int> &S) {
      set<int>::iterator q = S.begin();
      while (q!=S.end()) hasher.hash(*q++);
    }

  public:

    //---:---<*>---:---<*>---:---<*>---:---<*>---:---<*>
    // Makes a random tree with `s' siblings and `m' nodes.
    // The value at the node is in [0,M)
    void make_random_tree2(tree<int> &T,tree<int>::iterator n,
                           int M, int m,int s,
                           randomg_t &rnd) {
      if (!m) return;
      // Toma los m nodos y los divide en `ss' siblings donde s es aleatorio
      // en [1,s]
      int ss = rnd.rand()%s+1;
      // Inserta un nodo en la raiz
      n = T.insert(n,rnd.rand()%M);
      m--; // toma en cuenta que ya inserto 1
      // Reparte los nodos que quedan aleatoriamente en los ss hijos
      vector<int> v(ss,0);
      for (int j=0; j<m; j++) v[rnd.rand()%ss]++;
      // A esta altura tenemos un vector v[] con s enteros
      // cuya suma es `m'.  Algunos pueden ser nulos, pero no importa
      // porque en ese caso la llamada recursiva no va a hacer nada. 
      for (int j=0; j<v.size(); j++) 
        make_random_tree2(T,n.lchild(),M,v[j],s,rnd);
    }

    //---:---<*>---:---<*>---:---<*>---:---<*>---:---<*>
    // Wrapper
    void make_random_tree2(tree<int> &T,int M, int m,int s,
                           randomg_t &rnd) {
      T.clear();
      make_random_tree2(T,T.begin(),M,m,s,rnd);
    }

    void lisp2tree(string s, tree<int> &T) {
      unsigned int i = 0;
      lisp2tree(s, T, i, T.begin());
    }

    const char *i2s(int ok) { return (ok ? "OK" : "MAL"); }

    typedef bool (*is_recurrent_tree_t) (btree<int> &T,assoc_fun_t g);
    void verificar_irt(int &caso,int &okt,
                       is_recurrent_tree_t F,
                       string sT,
                       assoc_fun_t g,const char *gname,
                       int rvref,
                       int vrbs=0) {
      btree<int> T;
      int rvuser;
      lisp2btree(sT,T);

      rvuser = F(T,g);

      int ok = (rvuser == rvref);
      if (!ok || vrbs) {
        printf("---------\n");
        cout << "T: ";
        T.lisp_print();
        cout << endl;
        cout << "funcion assoc: " << gname << endl;
        cout << "ret val (user) " << rvuser << endl;
        cout << "ret val (ref) " << rvref << endl;
      }
      cout << "EJ-IRT|Caso" << caso++ << ". Estado: " << i2s(ok) << endl;
      okt += ok; 
    }

    //---:---<*>---:---<*>---:---<*>---:---<*>---:---<*>
    void eval_irt(is_recurrent_tree_t F,int vrbs=0) {
      cout << "----------------\nEvaluando ejercicio IS-RECURRENT-TREE" 
           << endl;
      int caso = 0, ok = 0;

#define VERIRT(T,g,rv) verificar_irt(caso,ok,F,T,g,#g,rv,vrbs)

      VERIRT("(5 (2 1 1) (3 1 (2 1 1)))",suma,1);
      VERIRT("(5 (3 (2 1 1) 1) (3 1 (2 1 1)))",suma,0);
      VERIRT("(5 (2 1 1) (3 1 (1 1 1)))",suma,0);

      VERIRT("(5 (3 3 2) (5 1 (5 4 5)))",max,1);
      VERIRT("(5 (3 3 2) (5 1 (5 4 3)))",max,0);

      VERIRT("(1 (2 3 2) (1 1 (4 4 5)))",min,1);
      VERIRT("(1 (2 3 2) (1 1 (4 4 3)))",min,0);

      int total = caso;
      printf("ESTADO EJ-IRTotal %d, ok %d, mal %d. Todos bien? %s\n",
             total,ok,total-ok,i2s(ok==total));
    }

    //---:---<*>---:---<*>---:---<*>---:---<*>---:---<*>
    // make BT full and recursive: n=g(left,right)
    void fix_tree(btree<int> &T,assoc_fun_t g,
                  btree<int>::iterator n,
                  int fix,randomg_t &rnd) {
      if (n==T.end()) {
        n = T.insert(n,0);
        return;
      }
      if (n.left()==T.end() && n.right()==T.end()) return;
      fix_tree(T,g,n.left(),fix,rnd);
      fix_tree(T,g,n.right(),fix,rnd);
      if (fix || rnd.rand()%10>5) *n = g(*n.left(),*n.right());
    }

#define CASE(j,fun) case j: f=fun; fname=#fun; break
    //---:---<*>---:---<*>---:---<*>---:---<*>---:---<*>
    int evalr_irt(is_recurrent_tree_t F,int seed,int vrbs=0) {
      int ncasos=20;
      hasher_t hasher;
      randomg_t rnd;
      rnd.reset(seed);
      int nfun=3;
      assoc_fun_t f;
      const char *fname=NULL;
      int retval;
      for (int jfun=0; jfun<nfun; jfun++) {
        switch (jfun) {
          CASE(0,suma);
          CASE(1,max);
          CASE(2,min);
        }
        
        for (int caso=0; caso<ncasos; caso++) {
          if (vrbs>=2) printf("Ejerc IRT. Caso %d\n",caso);
          int m = 20 + rnd.rand()%10;
          btree<int> T;
          make_random_btree(T,10,0.8);
          int fix = rnd.rand()%10<5;
          fix_tree(T,f,T.begin(),fix,rnd);

          retval = F(T,f);
          if (vrbs) {
            cout << "T: "; T.lisp_print(); cout << endl;
            cout << "assoc fun: " << fname << endl;
            cout << "ret val: " << retval << endl;
          }
          hasher.hash(retval);
        }
      }
      int H = hasher.val()%1000;
      if (vrbs) printf("EJ-IRT: SEED=%03d, HASH=%03d\n",seed,H);
      return H;
    }

    //---:---<*>---:---<*>---:---<*>---:---<*>---:---<*>
    typedef void (*ordered_tree_t)(vector<int> &V, btree<int> &T);
    void verificar_otr(int &caso,int &okt,
                       ordered_tree_t F,
                       string sV,
                       string sTref,
                       int vrbs=0) {
      vector<int> V;
      btree<int> Tuser,Tref;
      s2v(sV,V);
      lisp2btree(sTref,Tref);

      F(V,Tuser);

      int ok = (Tuser == Tref);
      if (!ok || vrbs) {
        printf("---------\n");
        cout << "V: "; dump(V); cout << endl;
        cout << "Tuser: "; Tuser.lisp_print(); cout << endl;
        cout << "Tref: "; Tref.lisp_print(); cout << endl;
      }
      cout << "EJ-OTR|Caso" << caso++ << ". Estado: " << i2s(ok) << endl;
      okt += ok; 
    }

    //---:---<*>---:---<*>---:---<*>---:---<*>---:---<*>
    void eval_otr(ordered_tree_t F,int vrbs=0) {
      cout << "----------------\nEvaluando ejercicio IS-RECURRENT-TREE" 
           << endl;
      int caso = 0, ok = 0;

#define VEROTR(V,T) verificar_otr(caso,ok,F,V,T,vrbs)

      VEROTR("1 5 10 8 7 2","(1 . (5 2 (10 (8 7 .) .)))");

      int total = caso;
      printf("ESTADO EJ-IRT Total %d, ok %d, mal %d. Todos bien? %s\n",
             total,ok,total-ok,i2s(ok==total));
    }


#define CASE(j,fun) case j: f=fun; fname=#fun; break
    //---:---<*>---:---<*>---:---<*>---:---<*>---:---<*>
    int evalr_otr(ordered_tree_t F,int seed,int vrbs=0) {
      int ncasos=20;
      hasher_t hasher;
      randomg_t rnd;
      rnd.reset(seed);

      for (int caso=0; caso<ncasos; caso++) {
        if (vrbs>=2) printf("Ejerc OTR. Caso %d\n",caso);
        int m = 20 + rnd.rand()%10;
        vector<int> V;
        btree<int> T;
        for (int j=0; j<m; j++) V.push_back(rnd.rand()%20);

        F(V,T);
        if (vrbs) {
          cout << "V: "; dump(V); cout << endl;
          cout << "T(user): "; T.lisp_print(); cout << endl;
        }
        hash(hasher,T);
      }
      int H = hasher.val()%1000;
      if (vrbs) printf("EJ-OTR: SEED=%03d, HASH=%03d\n",seed,H);
      return H;
    }

    //---:---<*>---:---<*>---:---<*>---:---<*>---:---<*>
    typedef void (*bijsubset_t)(set<int> &S, map_fun_t f, set<int> &S1);
    void verificar_bjs(int &caso,int &okt,
                       bijsubset_t F,
                       map_fun_t f,const char *fname,
                       string sin,string s1ref,
                       int vrbs=0) {
      set<int> Sin,S1ref,S1user;
      s2s(sin,Sin);
      s2s(s1ref,S1ref);

      F(Sin,f,S1user);

      int ok = (S1user == S1ref);
      if (!ok || vrbs) {
        printf("---------\n");
        cout << "S: " << sin << endl;
        cout << "funcion de mapeo: " << fname << endl;
        cout << "S1(user): "; print_set(S1user); cout << endl;
        cout << "S1(ref): "; print_set(S1ref); cout << endl;
      }
      cout << "EJ-BJS|Caso" << caso++ << ". Estado: " << i2s(ok) << endl;
      okt += ok; 
    }

    //---:---<*>---:---<*>---:---<*>---:---<*>---:---<*>
    void eval_bjs(bijsubset_t F,int vrbs=0) {
      cout << "----------------\nEvaluando ejercicio 1 (bijsubset_t)" << endl;
      int caso = 0, ok = 0;
      
#define VERBJS(f,s1,s2) \
      verificar_bjs(caso,ok,F,f,#f,s1,s2,vrbs);

      VERBJS(square,"1 -1 2","2");
      VERBJS(square,"1 -1 2 -3","2 -3");
      VERBJS(cube,"-1 1 -2 2","-1 1 -2 2");
      VERBJS(neg,"-1 1 -2 2","-1 1 -2 2");
      VERBJS(div2,"0 1 2 3 4 5","");
      VERBJS(div2,"0 2 5 6","0 2 5 6");

      int total = caso;
      printf("ESTADO EJ1: total %d, ok %d, mal %d. Todos bien? %s\n",
             total,ok,total-ok,i2s(ok==total));
    }

    //---:---<*>---:---<*>---:---<*>---:---<*>---:---<*>
    int evalr_bjs(bijsubset_t F,int seed,int vrbs=0) {
      int ncasos=20;
      hasher_t hasher;
      randomg_t rnd;
      rnd.reset(seed);
      int nfun=4;
      map_fun_t f;
      for (int jfun=0; jfun<4; jfun++) {
        switch (jfun) {
        case 0: f=square; break;
        case 1: f=cube; break;
        case 2: f=div2; break;
        case 3: f=neg; break;
        }
        
        for (int caso=0; caso<ncasos; caso++) {
          if (vrbs>=2) printf("Ejerc BJS. Caso %d\n",caso);
          int m = 20 + rnd.rand()%10;
          set<int> S,S1;
          for (int j=0; j<m; j++) {
            int x = -10 + rnd.rand()%10;
            S.insert(x);
          }
          F(S,f,S1);
          if (vrbs) {
#if 0
            printf("hash check: m %d, k %d\nT: ",m,k);
            T.lisp_print();
            printf("\n");
#endif
          }
          hash(hasher,S1);
        }
      }
      int H = hasher.val()%1000;
      if (vrbs) printf("EJ-BJS: SEED=%03d, HASH=%03d\n",seed,H);
      return H;
    }

    //---:---<*>---:---<*>---:---<*>---:---<*>---:---<*>
    typedef bool (*preimage_t)(vector< set<int> >&VX,map_fun_t f,
                  set<int> &Y,set<int> &preY);
    void verificar_pri(int &caso,int &okt,
                       preimage_t F,
                       string sVX,
                       map_fun_t f,const char *fname,
                       string sY,
                       string spreYref,
                       bool rvref,
                       int vrbs=0) {
      vector< set<int> > VX;
      set<int> Y,preYref,preYuser;
      bool rvuser;
      s2vs(sVX,VX);
      s2s(sY,Y);
      s2s(spreYref,preYref);

      rvuser = F(VX,f,Y,preYuser);

      int ok = (rvuser==rvref && preYuser == preYref);
      if (!ok || vrbs) {
        printf("---------\n");
        cout << "VX: " << sVX << endl;
        cout << "funcion de mapeo: " << fname << endl;
        cout << "Y: " << sY << endl;
        cout << "ret val(user): " << rvuser << endl;
        cout << "ret val(ref): " << rvref << endl;
        cout << "preY(user): "; print_set(preYuser); cout << endl;
        cout << "preY(ref): "; print_set(preYref); cout << endl;
      }
      cout << "EJ-PRI|Caso" << caso++ << ". Estado: " << i2s(ok) << endl;
      okt += ok; 
    }

    //---:---<*>---:---<*>---:---<*>---:---<*>---:---<*>
    void eval_pri(preimage_t F,int vrbs=0) {
      cout << "----------------\nEvaluando ejercicio preimage" << endl;
      int caso = 0, ok = 0;

    // void verificar_pri(int &caso,int &okt,
    //                    preimage_t F,
    //                    string sVX,
    //                    map_fun_t f,const char *fname,
    //                    string sY,
    //                    string spreYref,
    //                    bool rvref,
    //                    int vrbs=0) {

      // VX=[{2,3,4},{-1,1,2},{5,6,7},{-1,-2}] y Y={1,4}, y f(x)=x^2

#define VERPRI(vs,f,y,prey,rv)                            \
      verificar_pri(caso,ok,F,vs,f,#f,y,prey,rv,vrbs)

      // VX=[{2,3,4},{-1,1,2},{5,6,7},{-1,-2}]
      VERPRI("3 2 3 4 3 -1 1 2 3 5 6 7 2 -1 -2",square,"1 4","-1 1 2",1);
      VERPRI("3 2 3 4 3 -1 1 2 3 5 6 7 2 -1 -2",neg,"1 -1 -2","-1 1 2",1);
      VERPRI("3 2 3 4 3 -1 1 2 3 5 6 7 2 -1 -2",square,"1 4 5","",0);
      VERPRI("3 2 3 4 3 -1 1 2 3 5 6 7 2 -1 -2",cube,"-1 -8","-1 -2",1);
      VERPRI("3 2 3 4 3 -1 1 2 3 5 6 7 2 -1 -2",div2,"1 2","2 3 4",1);

      int total = caso;
      printf("ESTADO EJ-PRI: total %d, ok %d, mal %d. Todos bien? %s\n",
             total,ok,total-ok,i2s(ok==total));
    }


    //---:---<*>---:---<*>---:---<*>---:---<*>---:---<*>
    int evalr_pri(preimage_t F,int seed,int vrbs=0) {
      int ncasos=20;
      hasher_t hasher;
      randomg_t rnd;
      rnd.reset(seed);
      int nfun=4;
      map_fun_t f;
      const char *fname=NULL;
      for (int jfun=0; jfun<4; jfun++) {
        switch (jfun) {
          CASE(0,square);
          CASE(1,cube);
          CASE(2,div2);
          CASE(3,neg);
        }
        
        for (int caso=0; caso<ncasos; caso++) {
          if (vrbs>=2) printf("Ejerc PRI. Caso %d\n",caso);
          // VX size
          int nVX = 10;
          // S size
          int nS = 4;
          vector<set<int> > VX(nVX);
          for (int j=0; j<nVX; j++) {
            for (int k=0; k<nS; k++) {
              int x = -5+rnd.rand()%10;
              VX[j].insert(x);
            }
          }
          // Pick a random VX set
          int j = rnd.rand()%nVX;
          // Compute its image
          set<int> Y, &X= VX[j], preY;
          set<int>::iterator q = X.begin();
          while (q!=X.end()) Y.insert(f(*q++));
          // With 20% prob chane the image
          if (rnd.rand()%100<50) {
            Y.insert(-5+rnd.rand()%10);
          }
          int rv = F(VX,f,Y,preY);
          if (vrbs) {
            printf("----------------\n");
            print_vset(VX,"VX");
            cout << "f: " << fname << endl;
            print_set(Y,"Y");
            cout << "retval: " << rv << endl;
            print_set(preY,"preY");
          }
          hasher.hash(rv);
          hash(hasher,preY);
        }
      }
      int H = hasher.val()%1000;
      if (vrbs) printf("EJ-PRI: SEED=%03d, HASH=%03d\n",seed,H);
      return H;
    }

    //---:---<*>---:---<*>---:---<*>---:---<*>---:---<*>
    void dump(stack<int> S,string s="") {
      cout << s; 
      while (!S.empty()){
        cout << S.top() << " ";
        S.pop();
      }
      cout << endl;
    }

    //---:---<*>---:---<*>---:---<*>---:---<*>---:---<*>
    void dump(queue<int> Q,string s="") {
      cout << s; 
      while (!Q.empty()){
        cout << Q.front() << " ";
        Q.pop();
      }
      cout << endl;
    }
	
    //---:---<*>---:---<*>---:---<*>---:---<*>---:---<*>
    void s2sta(string s, stack<int> &S){
      istringstream is(s);
      int n;
      while (is >> n)
        S.push(n);
    }
	
    void random_shuffle(randomg_t &rnd,vector<int> &v,int k1,int k2) {
      int n = k2-k1;
      if (n<=1) return;
      int n1 = n/2, n2 = n-n1, j1=k1, j2=k1+n1;
      this->random_shuffle(rnd,v,k1,j2);
      this->random_shuffle(rnd,v,j2,k2);
      vector<int> w;
      while (n1 || n2) {
        int choose1 = (rnd.rand()%(n1+n2))<n1;
        if (choose1) { w.push_back(v[j1++]); n1--; }
        else { w.push_back(v[j2++]); n2--; }
      }
      for (int j=0; j<n; j++) v[k1+j] = w[j];
    }

    void random_shuffle(randomg_t &rnd,vector<int> &v) {
      this->random_shuffle(rnd,v,0,v.size());
    }

  };

}

#endif
